#include <gtk/gtk.h>
#include "iwcommon.h"
#include "iwmisc.h"		/* get and set funcs from iwconfig.c */
#include "xmisc.h"

int skfd = -1;		/* generic raw socket desc.	*/
struct wireless_info	info;
char wireless_device[4];

/* Callbacks */
void destroy(GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}

static void device_changed(GtkWidget* w, gpointer data)
{
	GtkWidget* window = (GtkWidget*)data;
	GtkWidget* device_combo = gtk_object_get_data(GTK_OBJECT(window), "device");
	GtkWidget* essid_entry = gtk_object_get_data(GTK_OBJECT(window), "essid");
	GtkWidget* nick_entry = gtk_object_get_data(GTK_OBJECT(window), "nick");
	GtkWidget* mode_combo = gtk_object_get_data(GTK_OBJECT(window), "mode");	
	
	strcpy(wireless_device, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(device_combo)->entry)));
	get_info(skfd, wireless_device, &info);
	
	gtk_entry_set_text(GTK_ENTRY(essid_entry), info.essid);
	gtk_entry_set_text(GTK_ENTRY(nick_entry), info.nickname);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mode_combo)->entry), operation_mode[info.mode]);
}

void set(GtkWidget *widget, gpointer data)
{
	GtkWidget* window = (GtkWidget*)data;
	GtkWidget* essid_entry = gtk_object_get_data(GTK_OBJECT(window), "essid");
	GtkWidget* nick_entry = gtk_object_get_data(GTK_OBJECT(window), "nick");
	GtkWidget* mode_combo = gtk_object_get_data(GTK_OBJECT(window), "mode");
	
	char *buffer;
	
	buffer = gtk_entry_get_text(GTK_ENTRY(essid_entry));
	if(!set_essid(skfd, wireless_device, buffer))
		return;
	
	buffer = gtk_entry_get_text(GTK_ENTRY(nick_entry));
	if(!set_nick(skfd, wireless_device, buffer))
		return;
	
	buffer = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(mode_combo)->entry));
	if(!set_mode(skfd, wireless_device, buffer))
		return;
	
	gtk_main_quit();
		
}

/* Put a list of wireless devices into list */
void devices_in_combo(GtkWidget *combo)
{
	char buff[1024];
  struct ifconf ifc;
  struct ifreq *ifr;
	GList *list = NULL;
  int i;

  /* Get list of active devices */
  ifc.ifc_len = sizeof(buff);
  ifc.ifc_buf = buff;
  if(ioctl(skfd, SIOCGIFCONF, &ifc) < 0)
	{
		fprintf(stderr, "SIOCGIFCONF: %s\n", strerror(errno));
		return;
	}
  ifr = ifc.ifc_req;

  /* add the wireless device to the glist */
	for(i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++)
	{
		if(get_info(skfd, ifr->ifr_name, &info) > -1)
			list = g_list_append(list, ifr->ifr_name);
	}
	/* get info on the first device again so it displays right */
	get_info(skfd, wireless_device, &info);
	gtk_combo_set_popdown_strings(GTK_COMBO(combo), list);
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry), wireless_device);	
}

void setup_gui(GtkWidget *window)
{
	GtkWidget *table;
	GtkWidget *mode, *device;
	GtkWidget *bok, *bcancel;
	GList *mode_strings = NULL;
	int i = 0;
	
	table = gtk_table_new(5, 2, TRUE);
	gtk_container_set_border_width(GTK_CONTAINER(table), 2);	
	
	/* Device */
	add_label(table, "Device", 0, 0);
	device = gtk_combo_new();
	gtk_combo_set_value_in_list(GTK_COMBO(device), TRUE, FALSE);
	devices_in_combo(device);
	gtk_object_set_data(GTK_OBJECT(window), "device", device);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(device)->entry), "changed",
					  GTK_SIGNAL_FUNC(device_changed), window);	
	gtk_table_attach(GTK_TABLE(table), device, 
					1, 3, 0, 1,	GTK_FILL, GTK_FILL, 2, 2);	

	add_label(table, "ESSID", 1, 0);
	add_entry(window, table, 1, "essid", info.essid);
	add_label(table, "Name", 2, 0);
	add_entry(window, table, 2, "nick", info.nickname);

	/* Mode */
	add_label(table, "Mode", 3, 0);
	mode = gtk_combo_new();
	gtk_combo_set_value_in_list(GTK_COMBO(mode), TRUE, FALSE);
	for(i=0; i<6; i++)
		mode_strings = g_list_append(mode_strings, operation_mode[i]);
	
	gtk_combo_set_popdown_strings(GTK_COMBO(mode), mode_strings);
	gtk_object_set_data(GTK_OBJECT(window), "mode", mode);	
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mode)->entry), operation_mode[info.mode]);
  gtk_table_attach(GTK_TABLE(table), mode, 
					1, 3, 3, 4,	GTK_FILL, GTK_FILL, 2, 2);

	/* Ok, Cancel Buttons */
	bok = gtk_button_new_with_label("Ok");
	gtk_signal_connect (GTK_OBJECT(bok), "clicked",
			GTK_SIGNAL_FUNC(set), window);
	gtk_table_attach(GTK_TABLE(table), bok, 
					1, 2, 4, 5,	GTK_FILL, GTK_FILL, 2, 2);

	bcancel = gtk_button_new_with_label("Cancel");
	gtk_signal_connect (GTK_OBJECT(bcancel), "clicked",
			GTK_SIGNAL_FUNC(destroy), NULL);
	gtk_table_attach(GTK_TABLE(table), bcancel, 
					2, 3, 4, 5,	GTK_FILL, GTK_FILL, 2, 2);

	/* Finish up */
	gtk_widget_show_all(table);
	gtk_container_add (GTK_CONTAINER (window), table);
	
}

/* Get the first wireless device to kick things off */
char *first_device()
{
	char buff[1024];
  struct ifconf ifc;
  struct ifreq *ifr;
  int i;

  /* Get list of active devices */
  ifc.ifc_len = sizeof(buff);
  ifc.ifc_buf = buff;
  if(ioctl(skfd, SIOCGIFCONF, &ifc) < 0)
	{
		sprintf(buff, "SIOCGIFCONF: %s\n", strerror(errno));
		fprintf(stderr, "%s\n", buff);
		gtk_message_box(buff, TRUE);
		return NULL;
	}
	ifr = ifc.ifc_req;
  /* add the wireless device to the glist */
	for(i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++)
	{
		if(get_info(skfd, ifr->ifr_name, &info) > -1)
			return ifr->ifr_name;
	}
	return NULL;
}

int main(int argc, char *argv[])
{
	GtkWidget *window;
	char error[32];
	char *dev = NULL;
	
	/* init gtk here so error messages can use it */
	gtk_init(&argc, &argv); 
	
  /* Create a channel to the NET kernel. */
  if((skfd = sockets_open()) < 0)
		return -1;
	
	dev = first_device();
	/* if we didn't get a wireless device... */
	if(!dev)
	{
		fprintf(stderr, "No Wireless Devices Found!\n");		
		strcpy(error, "No Wireless Devices Found!");				
		gtk_message_box(error, TRUE);
		return 0;
	}
	/* else, use the first one found */
	strcpy(wireless_device, dev);
	get_info(skfd, wireless_device, &info);
	
	/* gtk stuff */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy), NULL);

	setup_gui(window);
	gtk_widget_show(window);

	gtk_main();
	close(skfd);

	return 0;
}
